@@ -4,7 +4,7 @@
 # Pod stripped from pm file by OODoc 2.02.
 package Dancer2::Plugin::LogReport;
 use vars '$VERSION';
-$VERSION = '1.19';
+$VERSION = '1.20';
 
 
 use warnings;
@@ -146,9 +146,9 @@ on_plugin_import
 sub process($$)
 {   my ($dsl, $coderef) = @_;
     try { $coderef->() } hide => 'ALL', on_die => 'PANIC';
-    my $success = $@->died ? 0 : 1;
-    $@->reportAll(is_fatal => 0);
-    $success;
+	my $e = $@;  # fragile
+    $e->reportAll(is_fatal => 0);
+    $e->success || 0;
 }
 
 register process => \&process;
@@ -224,7 +224,7 @@ sub _forward_home($)
     # Don't forward if it's a GET request to the error page, as it will
     # cause a recursive loop. In this case, do nothing, and let dancer
     # handle it.
-    my $req = $dsl->app->request;
+    my $req = $dsl->app->request or return;
     return if $req->uri eq $page && $req->is_get;
 
     $dsl->redirect($page);
